library(tsDyn)
f <- function(x){# TAR model
  if(x>=0) return(0.9*x)
  if(x<0) return(0.5*x)
}

sim <- function(f,n){#simulation
  temp <- rep(NA,(n+100))
  temp[1] <- 1
  e <- rnorm((n+100),0,1)
  for (i in 1:((n-1)+100)){
    temp[i+1] <- f(temp[i])+e[i+1]
  }
  u <- temp[101:(100+n)]
  u
}

bootstrap=function(u,fhat,residuals){
  bb=rep(NA,(n+100))
  bb[1]=u[1]
  estar=sample(residuals,(n+101),replace=TRUE)
  for (i in 1:(n+99)){
    bb[i+1] <- fhat(bb[i])+estar[i+1]
  }
  b=bb[101:(n+100)]
  fv=fhat(u[n])+estar[(n+101)]#X_{n+1}^*
  list(b=b,fv=fv)
}

n=100
TAR.sim <- function(f,n){
  ################################simulate and model estimation##########################
  u <- sim(f,n)
  x <- u[1:(n-1)]
  y <- u[2:n]
  fit <- setar(u,m=1)
  if(abs(fit$coef[2])>=1 | abs(fit$coef[4])>=1){#unit root
    thres <- NULL
    fit <- lm(y~x) #no threshold
    if(abs(fit$coef[2])>=1){
      fit <- ar.yw(u,order.max=1)
      fhat <- function(x) fit$ar * (x-fit$x.mean) + fit$x.mean
    }else{
      fhat <- function(x) fit$coef[2] * x + fit$coef[1]
    }    
  }else{
    thres <- fit$coef[5]
    fhat <- function(x) predict(fit,x)    
  }
  
  
  
  yhat <- fhat(u[n]) #fitted value
  
  #fitted residuals
  fres=y-sapply(x,fhat)
  fres <- fres-mean(fres)
  sigmahat=sd(fres)#sigma
  #predictive residuals
  resid=rep(NA,(n-1))
  for (i in 1:(n-1)){
    xtemp=x[-i]
    ytemp=y[-i]
    if(is.null(thres)){#no threshold    
      tempfit <- lm(ytemp~xtemp)
      resid[i]=y[i]-predict(tempfit,data.frame(xtemp=x[i]))
    }else{#threshold model
      idx <-(xtemp-thres)*(x[i]-thres)>=0 
      xtemp <- xtemp[idx]#same side of x[i]
      ytemp <- ytemp[idx]
      tempfit <- lm(ytemp~xtemp)
      resid[i]=y[i]-predict(tempfit,data.frame(xtemp=x[i]))
    }           
  }
  pres=resid-mean(resid)
  
  
  ###########bootstrap#######################
  Ffstarhat <- rep(NA,1000)
  Ffstar <- rep(NA,1000)
  Fsfsd <- rep(NA,1000)
  for (i in 1:1000){
    result=bootstrap(u,fhat,fres)
    ustar=result$b
    xstar=ustar[1:(n-1)]
    ystar=ustar[2:n]
    fitstar <- setar(ustar,m=1)
    if(abs(fitstar$coef[2])>=1 | abs(fitstar$coef[4])>=1){
      fitstar <- lm(ystar~xstar)
      if(abs(fitstar$coef[2])>=1){
        fitstar <- ar.yw(ustar,order.max=1)
        fhatstar <- function(x) fitstar$ar * (x-fitstar$x.mean) + fitstar$x.mean
      }else{
        fhatstar <- function(x) fitstar$coef[2] * x + fitstar$coef[1]
      }    
    }else{
      thres <- fitstar$coef[5]
      fhatstar <- function(x) predict(fitstar,x)    
    }
    sigmastar <- sd(ystar-sapply(xstar,fhatstar))   
    Ffstarhat[i] <- fhatstar(u[n])
    Ffstar[i] <- result$fv
    Fsfsd[i] <- (Ffstar[i] -Ffstarhat[i])/sigmastar
  }
  
  Fpstarhat <- rep(NA,1000)
  Fpstar <- rep(NA,1000)
  Fspsd <- rep(NA,1000)
  for (i in 1:1000){
    result=bootstrap(u,fhat,pres)
    ustar=result$b
    xstar=ustar[1:(n-1)]
    ystar=ustar[2:n]
    fitstar <- setar(ustar,m=1)
    if(abs(fitstar$coef[2])>=1 | abs(fitstar$coef[4])>=1){
      thres <- NULL
      fitstar <- lm(ystar~xstar)
      if(abs(fitstar$coef[2])>=1){
        fitstar <- ar.yw(ustar,order.max=1)
        fhatstar <- function(x) fitstar$ar * (x-fitstar$x.mean) + fitstar$x.mean
      }else{
        fhatstar <- function(x) fitstar$coef[2] * x + fitstar$coef[1]
      }    
    }else{
      thres <- fitstar$coef[5]
      fhatstar <- function(x) predict(fitstar,x)    
    }
    sigmastar <- sd(ystar-sapply(xstar,fhatstar))
    
    Fpstarhat[i] <- fhatstar(u[n])
    Fpstar[i] <- result$fv
    Fspsd[i] <- (Fpstar[i] -Fpstarhat[i])/sigmastar
  }
  ##############################get the interval##########################################
  #Ff
  Ffdown=yhat+quantile(Ffstar-Ffstarhat,.025)
  Ffup=yhat+quantile(Ffstar-Ffstarhat,.975)
  Ffl=Ffup-Ffdown
  Ffdown90=yhat+quantile(Ffstar-Ffstarhat,.05)
  Ffup90=yhat+quantile(Ffstar-Ffstarhat,.95)
  Ffl90=Ffup90-Ffdown90
  #Fp
  Fpdown=yhat+quantile(Fpstar-Fpstarhat,.025)
  Fpup=yhat+quantile(Fpstar-Fpstarhat,.975)
  Fpl=Fpup-Fpdown
  Fpdown90=yhat+quantile(Fpstar-Fpstarhat,.05)
  Fpup90=yhat+quantile(Fpstar-Fpstarhat,.95)
  Fpl90=Fpup90-Fpdown90
  #FSf
  FSfdown=yhat+quantile(Fsfsd,.025)*sigmahat
  FSfup=yhat+quantile(Fsfsd,.975)*sigmahat
  FSfl=FSfup-FSfdown
  FSfdown90=yhat+quantile(Fsfsd,.05)*sigmahat
  FSfup90=yhat+quantile(Fsfsd,.95)*sigmahat
  FSfl90=FSfup90-FSfdown90
  #FSp
  FSpdown=yhat+quantile(Fspsd,.025)*sigmahat
  FSpup=yhat+quantile(Fspsd,.975)*sigmahat
  FSpl=FSpup-FSpdown
  FSpdown90=yhat+quantile(Fspsd,.05)*sigmahat
  FSpup90=yhat+quantile(Fspsd,.95)*sigmahat
  FSpl90=FSpup90-FSpdown90
  
  #1000 future values
  truevalue <- f(u[n]) + rnorm(1000)
  
  Ff=0
  Fp=0
  FSf=0
  FSp=0
  Ff90=0
  Fp90=0
  FSf90=0
  FSp90=0
  for (i in 1:1000){
    if((truevalue[i]<Ffup) & truevalue[i]>Ffdown)
      Ff=Ff+1
    if((truevalue[i]<Fpup) & truevalue[i]>Fpdown)
      Fp=Fp+1
    if((truevalue[i]<FSfup) & truevalue[i]>FSfdown)
      FSf=FSf+1
    if((truevalue[i]<FSpup) & truevalue[i]>FSpdown)
      FSp=FSp+1
    if((truevalue[i]<Ffup90) & truevalue[i]>Ffdown90)
      Ff90=Ff90+1
    if((truevalue[i]<Fpup90) & truevalue[i]>Fpdown90)
      Fp90=Fp90+1
    if((truevalue[i]<FSfup90) & truevalue[i]>FSfdown90)
      FSf90=FSf90+1
    if((truevalue[i]<FSpup90) & truevalue[i]>FSpdown90)
      FSp90=FSp90+1
    
  }
  
  Ffc=Ff/1000
  Fpc=Fp/1000
  FSfc=FSf/1000
  FSpc=FSp/1000
  Ffc90=Ff90/1000
  Fpc90=Fp90/1000
  FSfc90=FSf90/1000
  FSpc90=FSp90/1000
  
  list(Ffc=Ffc,Fpc=Fpc,FSfc=FSfc,FSpc=FSpc,Ffl=Ffl,Fpl=Fpl,FSfl=FSfl,FSpl=FSpl,
       Ffc90=Ffc90,Fpc90=Fpc90,FSfc90=FSfc90,FSpc90=FSpc90,Ffl90=Ffl90,Fpl90=Fpl90,
       FSfl90=FSfl90,FSpl90=FSpl90)
}

Ffc=rep(NA,500)
Fpc=rep(NA,500)
FSfc=rep(NA,500)  
FSpc=rep(NA,500)
Ffc90=rep(NA,500)
Fpc90=rep(NA,500)
FSfc90=rep(NA,500)
FSpc90=rep(NA,500)
Ffl=rep(NA,500)
Fpl=rep(NA,500)
FSfl=rep(NA,500)  
FSpl=rep(NA,500)
Ffl90=rep(NA,500)
Fpl90=rep(NA,500)
FSfl90=rep(NA,500)
FSpl90=rep(NA,500)
for (i in 1:500){
  set.seed(5*i)
  results=TAR.sim(f,n)
  Ffc[i]=results$Ffc
  Fpc[i]=results$Fpc
  FSfc[i]=results$FSfc
  FSpc[i]=results$FSpc
  Ffl[i]=results$Ffl
  Fpl[i]=results$Fpl
  FSfl[i]=results$FSfl
  FSpl[i]=results$FSpl
  Ffc90[i]=results$Ffc90
  Fpc90[i]=results$Fpc90
  FSfc90[i]=results$FSfc90
  FSpc90[i]=results$FSpc90
  Ffl90[i]=results$Ffl90
  Fpl90[i]=results$Fpl90
  FSfl90[i]=results$FSfl90
  FSpl90[i]=results$FSpl90
  save.image("C:/Users/Penny/Dropbox/time series/new sim/tar/tar_thres_normal50.RData")
  print(i)
}
print(mean(Ffc))
print(mean(Fpc))
print(mean(FSfc))
print(mean(FSpc))
print(mean(Ffl))
print(mean(Fpl))
print(mean(FSfl))
print(mean(FSpl))
print(sd(Ffl))
print(sd(Fpl))
print(sd(FSfl))
print(sd(FSpl))
print(mean(Ffc90))
print(mean(Fpc90))
print(mean(FSfc90))
print(mean(FSpc90))
print(mean(Ffl90))
print(mean(Fpl90))
print(mean(FSfl90))
print(mean(FSpl90))
print(sd(Ffl90))
print(sd(Fpl90))
print(sd(FSfl90))
print(sd(FSpl90))
